--[[	PhantomPlates
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

local LibThreatClassic=LibStub and LibStub("LibThreatClassic2",true);
local NamePlateObjects={};

----------------------------------
--[[	NamePlate Functions	]]
----------------------------------
local function NamePlateUF_UpdateThreat(self)
	local obj,unit=NamePlateObjects[self],self.displayedUnit;
	if not obj then return; end

	if not UnitIsPlayer(unit) then
		local status=LibThreatClassic and LibThreatClassic:UnitThreatSituation("player",unit);
		if status and status>0 then
			obj.ThreatTexture:SetVertexColor(LibThreatClassic:GetThreatStatusColor(status));
			obj.ThreatTexture:Show();
		else obj.ThreatTexture:Hide(); end
	else obj.ThreatTexture:Hide(); end
end

local function NamePlate_OnAdded(self,unit,driver)
	local obj=NamePlateObjects[self.UnitFrame];
	if not obj then return; end

	if not UnitIsPlayer(unit) then
		local classification=UnitClassification(unit);
		obj.BorderTexture:SetDesaturated(classification=="rare" or classification=="rareelite");
		if classification=="worldboss" or classification=="elite" then
			obj.ClassificationTexture:SetTexture("Interface\\Tooltips\\EliteNameplateIcon");
			obj.ClassificationTexture:Show();
		elseif classification=="rareelite" then
			obj.ClassificationTexture:SetTexture("Interface\\Tooltips\\RareEliteNameplateIcon");
			obj.ClassificationTexture:Show();
		else obj.ClassificationTexture:Hide(); end
	else obj.ClassificationTexture:Hide(); end

	NamePlateUF_UpdateThreat(self.UnitFrame);
end

----------------------------------
--[[	LibThreatClassic Setup	]]
----------------------------------
if LibThreatClassic then
	local LTCIdentifier={};--	CallbackHandler-1.0 can take any value as an identifier, same identifiers overwrite each other on the same events

	LibThreatClassic.RegisterCallback(LTCIdentifier,"Activate",function()
		for unitframe in next,NamePlateObjects do
			if unitframe.displayedUnit then NamePlateUF_UpdateThreat(unitframe); end
		end
	end);

	LibThreatClassic.RegisterCallback(LTCIdentifier,"Deactivate",function()
		for _,obj in pairs(NamePlateObjects) do
			obj.ThreatTexture:Hide();
		end
	end);

	LibThreatClassic.RegisterCallback(LTCIdentifier,"ThreatUpdated",function(event,unitguid,targetguid)
		for unitframe in next,NamePlateObjects do
			if unitframe.displayedUnit and targetguid==UnitGUID(unitframe.displayedUnit) then
				NamePlateUF_UpdateThreat(unitframe);
			end
		end
	end);

	LibThreatClassic:RequestActiveOnSolo();
end

------------------
--[[	Hooks	]]
------------------
hooksecurefunc(NamePlateDriverFrame,"OnNamePlateCreated",function(self,base)--	Hook Nameplate creation
	local unitframe=base.UnitFrame;

	local health=unitframe.healthBar:CreateFontString(nil,"OVERLAY");--"NumberFontNormalSmall");
	health:SetFont("Fonts\\ArialN.ttf",10,"THICKOUTLINE");--	Fonts are easier to read when made from scratch rather than resizing an inherited one
	health:SetPoint("LEFT",0,0);
	health:SetTextColor(0,1,0);

	local border; do--	Border is unnamed and not stored in its parent
		for _,region in ipairs({unitframe.healthBar.border:GetRegions()}) do
			if region.GetTexture and region:GetTexture()=="Interface/Tooltips/Nameplate-Border" then
				border=region; break;
			end
		end
	end

	local classification=unitframe.healthBar.border:CreateTexture(nil,"ARTWORK");
	classification:SetDrawLayer("ARTWORK",1);
	classification:SetSize(64,32);
	classification:SetPoint("LEFT",unitframe.healthBar.border,"RIGHT",-26,-3);
	classification:Hide();

	local threat=unitframe.healthBar.border:CreateTexture(nil,"BACKGROUND");
	threat:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Flash");
	threat:SetTexCoord(0,144/256,270/512,302/512);
	threat:SetPoint("CENTER",0,1);
	threat:SetSize(144,32);

	NamePlateObjects[unitframe]={
		HealthText=health;
		BorderTexture=border;
		ClassificationTexture=classification;
		ThreatTexture=threat;
	};
end);

hooksecurefunc(NamePlateBaseMixin,"OnAdded",NamePlate_OnAdded);

hooksecurefunc("CompactUnitFrame_UpdateHealth",function(self)--	This is a shared function with other UnitFrames
	if not NamePlateObjects[self] then return; end
	NamePlateObjects[self].HealthText:SetText(UnitHealth(self.displayedUnit));
end);

----------------------------------
--[[	Player NamePlate	]]
----------------------------------
local PlayerNamePlate=CreateFrame("Frame",nil,UIParent);
PlayerNamePlate:SetPoint("CENTER",0,-128);
PlayerNamePlate:SetSize(128,32);
PlayerNamePlate:Hide();

PlayerNamePlate:RegisterEvent("PLAYER_LOGIN");

PlayerNamePlate:RegisterEvent("PLAYER_REGEN_ENABLED");
PlayerNamePlate:RegisterEvent("PLAYER_REGEN_DISABLED");
PlayerNamePlate:RegisterUnitEvent("UNIT_HEALTH","player");
PlayerNamePlate:RegisterUnitEvent("UNIT_HEALTH_FREQUENT","player");

PlayerNamePlate:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_LOGIN" then
--		Creates UnitFrame and injects NamePlateBaseMixin
		NamePlateDriverFrame:OnNamePlateCreated(self);--	Creates UnitFrame
		self.UnitFrame.RaidTargetFrame:Hide();--	RaidTargetIcon is always hidden on player

--		Mimic NamePlateDriverFrame:OnNamePlateAdded()
		NamePlateDriverFrame:ApplyFrameOptions(self,"player");--	Applies layout
		self:OnAdded("player",NamePlateDriverFrame);--	Sets unit

--		NamePlateDriverFrame:OnUnitAuraUpdate("player");--	Defunct in classic
--		NamePlateDriverFrame:OnRaidTargetUpdate();--	Excludes player
	end

	if not InCombatLockdown() then
		self:SetShown(event=="PLAYER_REGEN_DISABLED"
			or UnitHealth("player")<UnitHealthMax("player")
		);
	end
end);
